;==============================================================================
;  ARCHCALC.LSP     Architectural Calculator Utility     Version 1.0
;
;  Written by Robert R. Keefer, AIA     Copyright (c)1990
;  Thanks to Scott Durkee for his patient assistance.
;
;  This program is distributed as Shareware, which authorizes you to
;  use it for a trial period of up to 21 days.  If you continue to use
;  the program after the trial period, please register it with the
;  author by sending a registration fee of $15.00 to:
;
;     Robert R. Keefer
;     8517 S. Drexel Avenue
;     Oklahoma City, Oklahoma   73159
;  
;  This function simulates a simple "feet-and-inch" calculator that accepts
;  input in standard AutoCAD Architectural units.  It provides addition,
;  subtraction, multiplication, division and conversion operations for linear
;  feet, inches, and fractions of inches, and for square foot values.  The
;  units in the current drawing must be set to #4-Architectural for proper
;  operation.
;
;==============================================================================
;
(defun *error* (msg)
   (princ msg)
   (princ)
)
(defun C:ARCHCALC (/ Stak Entry Func Num Val Sqft Help)
   (setq OK T
         Stak '() )                      ;set Stak to empty list
   (while OK
      (initget "+ - * / ** // /N S L Q C H D ") ;set operation characters
      (princ "\n [Special Operations: ** (LfxLf=Sf) // (Sf/Lf=Lf) /N (f/f=#) D (decimal ft) ]")
      (setq Entry 
         (getdist 
            (strcat "\n Clear/Quit/Help/Linft/Sqft/operation above/ + - * /, or number : "))
            Func nil)
      (cond                              ;check for entry, clear or quit
         ((equal Entry nil)              ;if Entry nil, set OK nil to end func
            (setq OK nil))
         ((or (equal (type Entry) 'REAL)(equal (type Entry) 'INT))
            (setq Stak (cons Entry Stak)) ) 
         ((equal Entry "C")              ;Clear sets Stak to empty list
            (setq Stak '()               ;and sets Sqft nil
                  Sqft nil)
            (princ " ARCHCALC:  NUMBER STACK CLEARED TO 0 "))
         ((equal Entry "Q")
            (setq OK nil))
         (T (setq Func Entry))           ;if Entry is K_word, set Func to K_word
      )                                  ;end Cond
      (if Func                           ;if a Func is entered
         (progn                          ;progn to allow "then" to have argument
            (cond                        ;for each operation having K_word
               ((equal Func "+")         ;Addition operation
                  (setq Num (car (reverse Stak))
                        Stak (cdr (reverse Stak)))
                  (if Sqft               ;if Sqft is T, factor by 12 & return SF
                     (progn
                        (setq Num (* 12 Num))
                        (foreach Val Stak (setq Num (+ Num Val))) 
                        (princ (strcat " ARCHCALC:  TOTAL AREA =  " 
                                       (rtos (/ Num 12) 2 4) " square feet"))
                        (setq Stak (list (/ Num 12))))
                     (progn              ;if Sqft is F, return LF
                        (foreach Val Stak (setq Num (+ Num Val))) 
                        (setq Stak (list Num))   ;push Answer onto Stack
                        (princ (strcat " ARCHCALC:  SUM =  " 
                                       (rtos Num 4 6)))))
               )
               ((equal Func "-")         ;Subtraction operation
                  (setq Num (car (reverse Stak))
                        Stak (cdr (reverse Stak)))
                  (if Sqft               ;if Sqft is T, factor by 12 & return SF
                     (progn
                        (setq Num (* 12 Num))
                        (foreach Val Stak (setq Num (- Num Val))) 
                        (princ (strcat " ARCHCALC:  REMAINING AREA =  " 
                                       (rtos (/ Num 12) 2 4) " Square Feet"))
                        (setq Stak (list (/ Num 12))))
                     (progn              ;if Sqft is F, return LF
                        (foreach Val Stak (setq Num (- Num Val))) 
                        (setq Stak (list Num))
                        (princ (strcat " ARCHCALC:  REMAINDER =  " 
                                       (rtos Num 4 6)))))
               )
               ((equal Func "*")
                  (setq Num (car (reverse Stak))
                        Stak (cdr (reverse Stak)))
                  (foreach Val Stak (setq Num (* Num Val))) 
                  (setq Stak (list Num))
                  (if Sqft
                     (princ (strcat " ARCHCALC:  MULTIPLIED AREA =  " 
                                    (rtos Num 2 4) " Square Feet"))
                     (princ (strcat " ARCHCALC:  MULTIPLIED TOTAL =  " 
                                    (rtos Num 4 6))))
               )
               ((equal Func "/")
                  (setq Num (car (reverse Stak))
                        Stak (cdr (reverse Stak)))
                  (foreach Val Stak (setq Num (/ Num Val))) 
                  (setq Stak (list Num))
                  (if Sqft
                     (princ (strcat " ARCHCALC:  DIVIDED AREA =  " 
                                    (rtos Num 2 4) " Square Feet"))
                     (progn
                        (setq Sqft nil)
                        (princ (strcat " ARCHCALC:  DIVIDED TOTAL =  " 
                                       (rtos Num 4 6)))))
               )
               ((equal Func "**")
                  (if (> (length Stak) 2)
                     (princ " ERROR: PRESS "C" TO RESTART (SqFt division limited to 2 numbers) ")
                     (progn
                        (setq Num (car (reverse Stak))
                              Stak (cdr (reverse Stak))
                              Sqft T)
                        (foreach Val Stak (setq Num (/ (* Num Val) 144))) 
                        (setq Stak (list Num))
                        (if Sqft
                           (princ (strcat " ARCHCALC:  AREA =  " 
                                          (rtos Num 2 4) " Square Feet"))
                           (princ (strcat " Answer is:  " 
                                          (rtos Num 2 4) " ' ")))))
               )
               ((equal Func "//")
                  (if (> (length Stak) 2)
                     (princ " ERROR: PRESS "C" TO RESTART (SqFt division limited to 2 numbers) ")
                     (progn
                        (setq Num (car (reverse Stak))
                              Stak (cdr (reverse Stak))
                              Sqft nil)
                        (foreach Val Stak (setq Num (* (/ Num Val) 144))) 
                        (setq Stak (list Num))
                        (princ (strcat " ARCHCALC:  LENGTH =  " 
                                       (rtos Num 4 6)))))
               )
               ((equal Func "/N")
                  (if (> (length Stak) 2)
                     (princ " ERROR: PRESS C TO RESTART (SqFt division limited to 2 numbers) ")
                     (progn
                        (setq Num (car (reverse Stak))
                              Stak (cdr (reverse Stak)))
                        (foreach Val Stak (setq Num (/ Num Val))) 
                        (setq Stak (list Num))
                        (if Sqft
                           (progn
                              (princ (strcat " ARCHCALC:  ANSWER =  " 
                                             (rtos (* 12 Num) 2 4)))
                              (setq Sqft nil)
                              (setq Stak (list (* Num 12))))
                           (progn
                              (princ (strcat " ARCHCALC:  ANSWER =  " 
                                             (rtos Num 2 4)))
                              (setq Sqft nil)))))
               )
               ((equal Func "S")
                  (setq Num (car (reverse Stak))
                        Stak (cdr (reverse Stak))
                        Sqft T)
                  (princ (strcat " ARCHCALC:  LF NUMBER CONVERTED TO  " 
                                 (rtos (/ Num 12) 2 4) " Square Feet "))
                  (setq Stak (list (/ Num 12)))
               )
               ((equal Func "L")
                  (setq Num (car (reverse Stak))
                        Stak (cdr (reverse Stak))
                        Sqft nil)
                  (princ (strcat " ARCHCALC:  SF NUMBER CONVERTED TO  " 
                                 (rtos (* Num 12) 4 6)))
                  (setq Stak (list (* Num 12)))
               )
               ((equal Func "H")
                  (princ "\n HELP SCREEN AVAILABLE TO REGISTERED USERS ONLY!")
               )
               ((equal Func "D")
                  (setq Num (car (reverse Stak)))
                  (princ (strcat " ARCHCALC:  LF NUMBER TO DECIMAL =  " 
                                 (rtos (/ Num 12) 2 4) " Feet ")))
            )
         )
      )
   )
   (if Help
      (command "REDRAW" )
      (princ))
   (princ " QUITTING ARCHCALC CALCULATOR . . .  ")
   (princ)
)
